/*
* This is the VSI VS1011E sound driver
*/

#include <built_in.h>

extern void UART2_Write_Line(char *uart_text);
extern void UART2_Write_Long_Variable(long var);
 
// MP3 connections
sbit Mmc_Chip_Select           at LATG9_bit;
sbit MP3_CS                    at RG12_bit;
sbit MP3_RST                   at RG13_bit;
sbit DREQ                      at RG1_bit;
sbit BSYNC                     at RG14_bit;

sbit Mmc_Chip_Select_Direction at TRISG9_bit;
sbit MP3_CS_Direction          at TRISG12_bit;
sbit MP3_RST_Direction         at TRISG13_bit;
sbit DREQ_Direction            at TRISG1_bit;
sbit BSYNC_Direction           at TRISG14_bit;

// VS1011E constants
const char WRITE_CODE  = 0x02;
const char READ_CODE   = 0x03;
const char SCI_MODE_ADDR   = 0x00;
const char SCI_CLOCKF_ADDR = 0x03;
const char SCI_VOL_ADDR    = 0x0B;
const char SCI_BASS_ADDR = 0x02;

char  data_buffer_32[32];
char  volume_left, volume_right;
unsigned long i, file_size;

//Mute sound flag
extern int muteSound;

//const BUFFER_SIZE = 2048;
//const BUFFER_SIZE = 512;
const BUFFER_SIZE = 512;
char  BufferLarge[BUFFER_SIZE];

//Indicates if a MP3 file has been loaded from the MMC card
char file_loaded = 0;

//Indicates the current file position after running play_MP3_Chunk
long current_pos = 0;

//flag to indicate that the current song is done playing
int play_next_song = 0;

//song counter
int song_count = 0;

long unable_to_play = 0;

// Write text and goto new line CR and LF
extern void UART2_Write_Line (char *text);
extern void UART2_Write_Long_Variable(long var);

// Writes one word to MP3 SCI
void MP3_SCI_Write(char address, unsigned int data_in) {
  BSYNC = 1;

  MP3_CS = 0;                             // select MP3 SCI
  SPI2_Write(WRITE_CODE);
  SPI2_Write(address);
  SPI2_Write(Hi(data_in));                // high byte
  SPI2_Write(Lo(data_in));                // low byte
  MP3_CS = 1;                             // deselect MP3 SCI
  while (DREQ == 0);                      // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI
}

// Reads words_count words from MP3 SCI
void MP3_SCI_Read(char start_address, char words_count, unsigned int *data_buffer) {
  unsigned int temp;

  BSYNC = 1;

  MP3_CS = 0;                             // select MP3 SCI
  SPI2_Write(READ_CODE);
  SPI2_Write(start_address);

  while (words_count--) {                 // read words_count words byte per byte
    temp = SPI2_Read(0);
    temp <<= 8;
    temp += SPI2_Read(0);
    *(data_buffer++) = temp;
    while (DREQ == 0);                      // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI
  }
  MP3_CS = 1;                             // deselect MP3 SCI

}

// Write one byte to MP3 SDI
void MP3_SDI_Write(char data_) {

  MP3_CS = 1;
  BSYNC = 0;

  while (DREQ == 0);                      // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI

  SPI2_Write(data_);
  BSYNC = 1;
}

// Write 32 bytes to MP3 SDI
void MP3_SDI_Write_32(char *data_) {
  char i;

  MP3_CS = 1;
  BSYNC = 0;

  while (DREQ == 0);                      // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI

  for (i=0; i<32; i++)
  SPI2_Write(data_[i]);
  BSYNC = 1;
}

// Set volume
void Set_Volume(char left,char right) {
  unsigned int volume;

  volume = (left<<8) + right;             // calculate value
  MP3_SCI_Write(SCI_VOL_ADDR, volume);        // Write value to VOL register
}
// If volume Buttons are pressed update volume

void MP3_Init() {
//  ADPCFG = 0xFFFF;        // set all AN pins to digital
    
    
    
  UART2_Write_Line("Initializing VS1011E decoder interface");



  BSYNC = 1;
  MP3_CS = 1;

  // Hardware reset
  MP3_RST = 0;
  Delay_ms(10);
  MP3_RST = 1;

  while (DREQ == 0);

  MP3_SCI_Write(SCI_MODE_ADDR, 0x0800);
  MP3_SCI_Write(SCI_BASS_ADDR, 0x7A00);
  MP3_SCI_Write(SCI_CLOCKF_ADDR, 0x2000); // default 12 288 000 Hz


  volume_left  = 10;                      // Set volume to initial value
  volume_right = 10;
  Set_Volume(volume_left,volume_right);
}

void Init_Sound() {
  ADPCFG = 0xFFFF;        // set all AN pins to digital

  MP3_CS_Direction  = 0;                  // Configure MP3_CS as output
  MP3_CS            = 1;                  // Deselect MP3_CS
  MP3_RST_Direction = 0;                  // Configure MP3_RST as output
  MP3_RST           = 1;                  // Set MP3_RST pin

  DREQ_Direction  = 1;                    // Configure DREQ as input
  BSYNC_Direction = 0;                    // Configure BSYNC as output
  BSYNC           = 0;                    // Clear BSYNC
  BSYNC           = 1;                    // Clear BSYNC
  
  UART2_Write_Line("Initializing SPI");
  SPI2_Init_Advanced(_SPI_MASTER, _SPI_8_BIT, _SPI_PRESCALE_SEC_4, _SPI_PRESCALE_PRI_4,
                     _SPI_SS_DISABLE, _SPI_DATA_SAMPLE_MIDDLE, _SPI_CLK_IDLE_HIGH, _SPI_ACTIVE_2_IDLE);
}

void Load_MP3_File(char *filename){

  UART2_Write_Line("Initializing MMC_FAT");
  
  //Reset current position
  current_pos = 0;

  if (Mmc_Fat_Init() == 0) {
    if (Mmc_Fat_Assign(filename, 0) ) {
      UART2_Write_Line("Loaded MP3 File:");
      UART2_Write_Line(filename);
      file_loaded = 1;
    }
    else {
      UART2_Write_Line("Failed to load MP3 File: ");
      UART2_Write_Line(filename);
      file_loaded = 0;
    }

  }
  else {
    UART2_Write_Line("MMC FAT not initialized");
    file_loaded = 0;
  }

}



void Play_MP3_Chunk(){
long starting_pos = 0;

starting_pos = current_pos;

//Check if the sound has been muted
if(!muteSound){
    //Check if the MP3 was loaded from the MMC card
    if(file_loaded && ( file_size > current_pos)) {

     if(current_pos == 0){
      UART2_Write_Line("Play started.");
      Mmc_Fat_Reset(&file_size);          // Call Reset before file reading,
                                          //   procedure returns size of the file
      UART2_Write_Line("File Size:");
      UART2_Write_Long_Variable(file_size);
      play_next_song = 0;
      
      unable_to_play = 0;
      }

      //Run the while loop until the buffer is full
      while( (current_pos - starting_pos) < (BUFFER_SIZE) ){

        for (i=0; i<BUFFER_SIZE; i++) {
          Mmc_Fat_Read(BufferLarge + i);
        }

        for (i=0; i<BUFFER_SIZE/32; i++) {
          MP3_SDI_Write_32(BufferLarge + i*32);
        }

        current_pos += BUFFER_SIZE;


        //MP3 file reading loop
        //UART2_Write_Text(".");
      }

    //UART2_Write_Line("Current Pos:");
    //UART2_Write_Long_Variable(current_pos);
    
    //UART2_Write_Line("Play chunk finished.");
    }
    else{
      //MP3 File wasn't loaded
      //limit how many times the same error is printed
      if(unable_to_play < 3){
      UART2_Write_Line("Unable to play MP3 file.");
      }
      //reset mp3 position
      current_pos = 0;
      file_loaded = 0;
      play_next_song = 1;
      song_count++;
      
      unable_to_play++;
    }
 }
 
}









void Play_MP3(){

  if(file_loaded) {

    UART2_Write_Line("Play started.");
    Mmc_Fat_Reset(&file_size);          // Call Reset before file reading,
                                      //   procedure returns size of the file
    // send file blocks to MP3 SDI
    while (file_size > BUFFER_SIZE) {

      for (i=0; i<BUFFER_SIZE; i++) {
        Mmc_Fat_Read(BufferLarge + i);
      }

      for (i=0; i<BUFFER_SIZE/32; i++) {
        MP3_SDI_Write_32(BufferLarge + i*32);
      }

      file_size -= BUFFER_SIZE;


      //main update loop
      //UART2_Write_Text(".");
    }

    // send the rest of the file to MP3 SDI
    for (i=0; i<file_size; i++) {
      Mmc_Fat_Read(BufferLarge + i);
    }
    for (i=0; i<file_size; i++) {
      MP3_SDI_Write(BufferLarge[i]);
    }


    UART2_Write_Line("Play finished.");
  }
  else{
    //MP3 File wasn't loaded
    UART2_Write_Line("Unable to play MP3 file.");
  }

}